@@ -1,27 +1,24 @@
-#coding:utf-8
+# coding: utf-8
+u"""
+Основные объекты библиотеки: механизмы проверки прав, экшены, паки,
+контроллеры, кэш контроллеров.
 """
-Основные объекты библиотеки: механизмы проверки прав, экшены, паки, контроллеры, кэш контроллеров
-+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-"""
-import abc
-from functools import wraps
-import inspect
-import importlib
-import threading
+
+from __future__ import absolute_import
+from __future__ import print_function
 import re
+import abc
+import inspect
+import threading
 import warnings
+from functools import wraps
+from importlib import import_module
+from logging import getLogger
 
 from django import http
 from django.conf import settings
-from django.utils.importlib import import_module
 
-try:
-    from django.utils.log import logger
-except ImportError:
-    from django.utils.log import getLogger
-    logger = getLogger('django')
-
-from results import (
+from .results import (
     ActionResult,
     PreJsonResult,
     JsonResult,
@@ -32,8 +29,7 @@ from results import (
     OperationResult,
     ActionRedirectResult
 )
-
-from exceptions import (
+from .exceptions import (
     ApplicationLogicException,
     ActionException,
     ActionNotFoundException,
@@ -41,10 +37,7 @@ from exceptions import (
     ReinitException,
     ActionUrlIsNotDefined
 )
-
-import utils
-
-from context import (
+from .context import (
     ActionContext,
     ActionContextDeclaration,
     DeclarativeActionContext,
@@ -52,13 +45,14 @@ from context import (
     ContextBuildingError,
     CriticalContextBuildingError,
 )
-
-from metrics import create_statsd_client
+from .metrics import create_statsd_client
+from . import utils
+import six
+from six.moves import map
 
 _STATSD_CLIENT = create_statsd_client(settings)
-
-
 ACD = ActionContextDeclaration
+logger = getLogger('django')
 
 
 _clean_url = lambda s: re.sub(r'^[/^]*(.*?)[$/]*$', r'\1', s)
@@ -72,7 +66,7 @@ def _import_by_path(path):
     if match is None:
         raise ValueError(u'Wrong path to import: %r' % path)
     module_name, obj_name = match.groups()
-    module = importlib.import_module(module_name)
+    module = import_module(module_name)
     return getattr(module, obj_name)
 
 
@@ -133,7 +127,7 @@ def _must_be_replaced_by(alternative):
             warnings.warn(
                 "'{clazz}.{method}' deprecated! Use '{alternative}'!".format(
                     clazz=class_name,
-                    method=fn.func_name,
+                    method=fn.__name__,
                     alternative=alternative
                 ), FutureWarning, 2)
             return fn(self, *args, **kwargs)
@@ -147,11 +141,10 @@ def _must_be_replaced_by(alternative):
 #==============================================================================
 
 
-class AbstractPermissionChecker(object):
+class AbstractPermissionChecker(six.with_metaclass(abc.ABCMeta, object)):
     """
     Абстрактный механизм проверки прав
     """
-    __metaclass__ = abc.ABCMeta
 
     @abc.abstractmethod
     def has_action_permission(self, request, action, subpermission=None):
@@ -726,7 +719,7 @@ class ActionPack(object):
                 pack = pack.parent
         else:
             path = [pack.url]
-        return '/'.join([pack.controller.url] + map(_clean_url, path))
+        return '/'.join([pack.controller.url] + list(map(_clean_url, path)))
 
     def get_permission_code(self):
         # метод оставлен для совместимости
@@ -902,8 +895,8 @@ class ActionController(object):
         """
         self._actions_by_name.clear()
         self._actions_by_type.clear()
-        for full_path, v in self._url_patterns.iteritems():
-            _, action = v
+        for full_path, v in six.iteritems(self._url_patterns):
+            _, action, suff = v
             self._add_action_to_search_dicts(action, full_path)
         # Обновлять _packs_by_name и _packs_by_type не нужно!
 
@@ -996,7 +989,7 @@ class ActionController(object):
             raise
         except ContextBuildingError as e:
             # некритичную ошибку - показываем пользователю
-            return OperationResult.by_message(unicode(e))
+            return OperationResult.by_message(six.text_type(e))
         except RequiredFailed as e:
             # если контекст неправильный, то возвращаем
             # фейльный результат операции
@@ -1014,27 +1007,10 @@ class ActionController(object):
         self.process_context(request, context, action)
 
         try:
-            # Все ПРЕ обработчики
-            for pack in stack:
-                result = pack.pre_run(request, context)
-                if result is not None:
-                    return result
-            # Сам экшен
-            result = action.pre_run(request, context)
-            if result is not None:
-                return result
-            if suffix is None:
-                response = action.run(request, context)
-            else:
-                response = action.run(request, context, suffix)
-            result = action.post_run(request, context, response)
-            if result is not None:
-                return result
-            # Все ПОСТ обработчики с конца
-            for pack in reversed(stack):
-                result = pack.post_run(request, context, response)
-                if result is not None:
-                    return result
+            response, force = self._process_action(
+                action, request, context, stack, suffix=suffix)
+            if force:
+                return response
         except ApplicationLogicException as exc:
             return OperationResult(
                 success=False, message=exc.exception_message)
@@ -1042,8 +1018,37 @@ class ActionController(object):
         # по возможности запихиваем текущий контекст в response
         if isinstance(response, BaseContextedResult):
             response.set_context(context)
+
         return response
 
+    def _process_action(self, action, request, context, stack, suffix=None):
+        """
+        Вызов экшена вместе со всеми pre_run и post_run
+        """
+        # Все ПРЕ обработчики
+        for pack in stack:
+            result = pack.pre_run(request, context)
+            if result is not None:
+                return result, True
+        # Сам экшен
+        result = action.pre_run(request, context)
+        if result is not None:
+            return result, True
+        if suffix is None:
+            response = action.run(request, context)
+        else:
+            response = action.run(request, context, suffix)
+        result = action.post_run(request, context, response)
+        if result is not None:
+            return result, True
+        # Все ПОСТ обработчики с конца
+        for pack in reversed(stack):
+            result = pack.post_run(request, context, response)
+            if result is not None:
+                return result, True
+
+        return response, False
+
     def process_request(self, request):
         """
         Обработка входящего запроса *request* от клиента.
@@ -1113,7 +1118,7 @@ class ActionController(object):
         return self._find_action(type)
 
     def _find_pack(self, pack):
-        if isinstance(pack, basestring):
+        if isinstance(pack, six.string_types):
             inst = self._packs_by_name.get(pack)
         elif inspect.isclass(pack):
             inst = self._packs_by_type.get(pack)
@@ -1122,7 +1127,7 @@ class ActionController(object):
         return inst
 
     def _find_action(self, action):
-        if isinstance(action, basestring):
+        if isinstance(action, six.string_types):
             inst, _ = self._actions_by_name.get(action, (None, None))
         elif inspect.isclass(action):
             inst, _ = self._actions_by_type.get(action, (None, None))
@@ -1246,8 +1251,8 @@ class ActionController(object):
         left_packs = []
         right_packs = []
 
-        for url, value in self._url_patterns.iteritems():
-            packs_list, final_action = value
+        for url, value in six.iteritems(self._url_patterns):
+            packs_list, final_action, suff = value
 
             # Поиск пака и соседей в списке
             left_pack = None
@@ -1271,7 +1276,7 @@ class ActionController(object):
 
             # Создание нового урла
             full_path = self._build_full_path(packs_list, final_action)
-            new_patterns[full_path] = (packs_list[:], final_action)
+            new_patterns[full_path] = (packs_list[:], final_action, suff)
             current_packs_slice = packs_list[:pos + 1]
 
         self._url_patterns = new_patterns
@@ -1335,7 +1340,7 @@ class ActionController(object):
         new_patterns = {}
         current_packs_slice = None
 
-        for url, value in self._url_patterns.iteritems():
+        for url, value in six.iteritems(self._url_patterns):
             packs_list, final_action = value
 
             # Поиск исходного пака и экшена в нём
@@ -1375,11 +1380,11 @@ class ActionController(object):
         Отладочный метод.
         Выводит в консоль список всех адрес зарегистрированных в контроллере.
         '''
-        print '==== CONTROLLER WITH URL: %s ======' % self.url
+        print('==== CONTROLLER WITH URL: %s ======' % self.url)
         for key in sorted(self._url_patterns.keys()):
-            print key
-        print
-        print 'Total patterns %s' % len(self._url_patterns.keys())
+            print(key)
+        print()
+        print('Total patterns %s' % len(list(self._url_patterns.keys())))
 
     def get_action_by_url(self, url):
         """
@@ -1411,7 +1416,7 @@ class ActionController(object):
         '''
         Возвращение всех паков в контроллере
         '''
-        return self._packs_by_name.values()
+        return list(self._packs_by_name.values())
 
     def reset(self):
         '''
@@ -1465,7 +1470,7 @@ class ControllerCache(object):
     @classmethod
     def get_action_url(cls, type):
         """ Возвращает URL экшена *type* по его имени или классу """
-        assert isinstance(type, basestring) or issubclass(type, Action)
+        assert isinstance(type, six.string_types) or issubclass(type, Action)
         cls.populate()
         for cont in cls._controllers:
             url = cont.get_action_url(type)
@@ -1644,14 +1649,14 @@ class ControllerCache(object):
             for app_name in settings.INSTALLED_APPS:
                 try:
                     module = import_module('.app_meta', app_name)
-                except ImportError, err:
+                except ImportError as err:
                     if err.args[0].find('No module named') == -1:
                         raise
                     continue
 
                 # расширение словаря overridde'ов
                 for original_cls, new_inst in (
-                    getattr(module, 'action_pack_overrides', {}).iteritems()
+                    six.iteritems(getattr(module, 'action_pack_overrides', {}))
                 ):
                     if inspect.isclass(original_cls):
                         original_cls = original_cls.get_short_name()
@@ -1677,7 +1682,7 @@ class ControllerCache(object):
         Отладочный метод.
         Выводит в консоль адреса всех контроллеров зарегистрированных в кэше.
         '''
-        print '------------ CONTROLLER CACHE DUMP ------------'
+        print('------------ CONTROLLER CACHE DUMP ------------')
         for cont in cls._controllers:
             cont.dump_urls()
 
